#   Type: Python 3 script
# Author: Steven Sun <stevenlele@outlook.com>
#   Date: Feb 27, 2020
#  Notes: Put this script in "MediaInfoLib/Source/Resource/Text/Stream" and run.
#         It reads "Audio,General,Image,Menu,Other,Text,Video.csv" and generates
#         "Data_Comments.csv" in Excel dialect.

import csv

filenames = ('Audio.csv', 'General.csv', 'Image.csv',
             'Menu.csv', 'Other.csv', 'Text.csv', 'Video.csv')

csv.register_dialect('MediaInfo', delimiter=';')

index_key = 0
index_type_unit = 2
index_comment = 6

# initiate CSV header
comment_rows = [['Key', 'Type or Unit', 'Comment']]
comment_dict = {}

for file in filenames:
    with open(file, 'r', encoding='utf_8') as f:
        reader = csv.reader(f, dialect='MediaInfo')

        for row in reader:
            row_key = row[index_key]
            row_type_unit = row[index_type_unit]
            row_comment = row[index_comment]

            if row_key.endswith('/String'):
                # merge Key and Key/String
                row_key = row_key[:-7]
            elif row_key[:-1].endswith('/String'):
                # ignore Key/String1, 2, 3,...
                continue
            elif '/' in row_key:
                row_key = row_key.split('/')[-1]

            if row_key in comment_dict:
                # if row_key exists, append information
                if row_type_unit and row_type_unit not in comment_dict[row_key][0]:
                    comment_dict[row_key][0].append(row_type_unit)
                if row_comment and row_comment not in comment_dict[row_key][1]:
                    comment_dict[row_key][1].append(row_comment)
            elif row_type_unit or row_comment:
                # if row_key does not exist, add to dictionary
                # format: {'key': ([type_or_unit_1, ...], [comment_1, ...]), ...}
                list_type_unit = []
                list_comment = []
                if row_type_unit:
                    list_type_unit.append(row_type_unit)
                if row_comment:
                    list_comment.append(row_comment)
                comment_dict[row_key] = (list_type_unit, list_comment)

for key, value in sorted(comment_dict.items()):
    comment_rows.append([key, '; '.join(value[0]), '; '.join(value[1])])

with open('Data_Comments.csv', 'w', encoding='utf_8_sig', newline='') as f:
    writer = csv.writer(f)
    writer.writerows(comment_rows)



# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-

"""This plugin extends the autotools plugin to run from subdirectory in source tree.
In addition to the autotools and defaults keywords, this plugin uses the following
plugin-specific keyword:
    - make-directory:
      (string)
      Run autotools in the given path.
"""

import os.path

from snapcraft.plugins import autotools

class AutotoolsSubdirPlugin(autotools.AutotoolsPlugin):
    @classmethod
    def schema(cls):
        schema = super().schema()
        schema['properties']['make-directory'] = {
            'type': 'string',
            'default':  ''
        }

        return schema

    def __init__(self, name, options, project):
        super().__init__(name, options, project)
        if self.options.make_directory:
            self.builddir = os.path.join(self.build_basedir, self.options.make_directory)

    def enable_cross_compilation(self):
        return



#!/usr/bin/env python
import sys
import os
import urllib2
from urllib import urlencode
import cookielib
import re

#found at http://thebigbrowser.blogspot.com/2010/08/use-google-translate-from-console.html

# The google translate API can be found here (***NOT OPERATIONAL SINCE DECEMBER 2011***):
# http://code.google.com/apis/ajaxlanguage/documentation/#Examples

# Language codes are listed here:
#http://code.google.com/apis/ajaxlanguage/documentation/reference.html#LangNameArray

if len(sys.argv) < 3:
    name = os.path.basename(sys.argv[0])
    print '''
Usage:
    %s en es lovely spam
    %s es en < file.txt

Available language codes are listed here:
http://code.google.com/apis/ajaxlanguage/documentation/reference.html#LangNameArray

''' % (name,name)
    sys.exit(-1)

## hack to be able to display UTF-8 in Windows console

def fix_win32_console():
    ## set utf8 console
    if not sys.stdin.encoding == 'cp65001':
        os.system('chcp 65001 > nul')
    class UniStream(object):
        __slots__= "fileno", "softspace",
        def __init__(self, fileobject):
            self.fileno= fileobject.fileno()
            self.softspace = False
        def write(self, text):
            if isinstance(text, unicode):
                os.write(self.fileno, text.encode("utf_8"))
            else:
                os.write(self.fileno, text)
    sys.stdout= UniStream(sys.stdout)
    sys.stderr= UniStream(sys.stderr)

if sys.platform == "win32":
    fix_win32_console()

lang1=sys.argv[1]
lang2=sys.argv[2]

if len(sys.argv) > 3:
    text=' '.join(sys.argv[3:])
else:
    text=sys.stdin.read()

base_url='http://translate.google.com.br/translate_a/t'
# sample browser request
#http://translate.google.com/translate_a/t?client=t&text=col&hl=en&sl=en&tl=es&multires=1&otf=2&ssel=4&tsel=0&sc=1
params=urlencode({'client':'t',
    'text':text,
    'hl':'en',
    'sl':lang1,
    'tl':lang2,
    'otf':2,
    'multires':1,
    'ssel':0,
    'tsel':0,
    'sc':1,
    })

url=base_url + '?' + params

cookiejar = cookielib.CookieJar()
opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookiejar))
opener.addheaders = [('User-agent', 'Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1018.0 Safari/535.19'),
                    ('Referer', 'http://translate.google.com/')
]
response = opener.open(url)
translation=response.read()
matcher = re.search('\[\[\["(?P<human_readable_chunk>[^")]*)', translation)
sys.stdout.write(matcher.group('human_readable_chunk'))


#   Type: Python 3 script
# Author: Steven Sun <stevenlele@outlook.com>
#   Date: Feb 28, 2020
#  Notes: Put "Language.csv" and "Language_parsed.csv" in this folder
#         and run the script. It converts the parsed file back into the
#         original CSV, and generates a single language file for testing.

import csv
import sys
import os

#################################
######### Configuration #########

# edit the filename if the CSV file is placed elsewhere
language_file = 'Language.csv'
translated_file = 'Language_parsed.csv'

# output filename
output_file = 'Language_translated.csv'

######### Configuration #########
#################################

csv.register_dialect('MediaInfo', delimiter=';')

if not os.path.exists(language_file):
    print('Error: Language.csv file does not exist!')
    sys.exit(1)

if not os.path.exists(translated_file):
    print('Error: Translated file does not exist!')
    sys.exit(1)

output_rows = []
output_lang_rows = []
translated_rows = []

with open(translated_file, 'r', encoding='utf_8') as f:
    reader = csv.reader(f)

    header = next(reader)
    lang_code = header[2]
    for row in reader:
        key = row[0]
        translated = row[2]
        translated_rows.append((key, translated))

with open(language_file, 'r', encoding='utf_8') as f:
    reader = csv.reader(f, dialect='MediaInfo')

    header = next(reader)
    index_lang = header.index(lang_code)
    output_rows.append(header)
    output_lang_rows.append([header[0], header[index_lang]])
    column_count = len(header)

    for row in reader:
        key = row[0]
        translated_row = translated_rows.pop(0)
        if key != translated_row[0]:
            print(f'Error: Key "{key}" does not match the translated file "{translated_rows[0][0]}".')
            print('Please check whether you have changed the translated file or not.')
            sys.exit(1)
        translated_string = translated_row[1]
        if translated_string.startswith('"') and translated_string.endswith('"'):
            translated_string = translated_string.strip('"')
        new_row = row
        if len(new_row) < column_count:
            new_row.extend([''] * (column_count - len(new_row)))
        new_row[index_lang] = translated_string
        output_rows.append(new_row)
        output_lang_rows.append([key, translated_string])

with open(output_file, 'w', encoding='utf_8', newline='') as f:
    writer = csv.writer(f, dialect='MediaInfo')
    writer.writerows(output_rows)

with open(f'{lang_code}.csv', 'w', encoding='utf_8', newline='') as f:
    writer = csv.writer(f, dialect='MediaInfo')
    writer.writerows(output_lang_rows)

print('Info: Compile completed!')



#   Type: Python 3 script
# Author: Steven Sun <stevenlele@outlook.com>
#   Date: Feb 28, 2020
#  Notes: Put "Language.csv" in this folder and run the script.
#         It generates "Language_parsed.csv" for translation.
#         Download the latest file from "MediaInfo/Source/Resource".
#   Date: Mar 1, 2020
# Update: Fix bug in note adding (keyword mode).

import csv
import sys
import os

#################################
######### Configuration #########

# edit the filename if the CSV file is placed elsewhere
language_file = 'Language.csv'

# output filename
output_file = 'Language_parsed.csv'

# enter your language codes here
# it should be one in
# ar;be;bg;ca;cs;da;de;es;eu;fa;fr;gl;gr;hu;it;ja;ko;lt;nl;
# pl;pt;pt-BR;ro;ru;sk;sq;sv;th;tr;uk;zh-CN;zh-HK;zh-TW;hr;hy;ka
# the first one is target language
# the others are reference languages (if any)
# English (en) is the default source language
lang_codes = ['zh-CN', 'ja']

# comments file
comments_file = 'Data_Comments.csv'

# notes file
notes_file = 'Data_Notes.csv'

######### Configuration #########
#################################

csv.register_dialect('MediaInfo', delimiter=';')

if not os.path.exists(language_file):
    print('Error: Language.csv file does not exist!')
    sys.exit(1)

if not lang_codes:
    print('Error: No language code is specified!')
    sys.exit(1)

dict_comments = {}
if os.path.exists(comments_file):
    with open(comments_file, 'r', encoding='utf_8_sig') as f:
        reader = csv.reader(f)
        next(reader)  # skip header
        for row in reader:
            # key, type or unit, comment
            dict_comments[row[0]] = [row[1], row[2]]
else:
    print('Info: Comments file does not exist. Ignoring comments.')

notes = False
dict_notes_key = {}
dict_notes_keyword = {}
dict_notes_key_keyword = {}
if os.path.exists(notes_file):
    notes = True
    with open(notes_file, 'r', encoding='utf_8_sig') as f:
        reader = csv.reader(f)
        next(reader)  # skip header
        for row in reader:
            mode = row[1].lower().strip()
            if mode == '' or mode == 'keyword':
                dict_notes_keyword[row[0].lower().strip()] = row[2]
            elif mode == 'key':
                dict_notes_key[row[0]] = row[2]
            elif mode == 'key_keyword':
                dict_notes_key_keyword[row[0].strip()] = row[2]
else:
    print('Info: Notes file does not exist. Ignoring notes.')

output_rows = []

with open(language_file, 'r', encoding='utf_8') as f:
    reader = csv.reader(f, dialect='MediaInfo')

    header = next(reader)
    index_lang = []
    lang_codes.insert(0, 'en')
    for lang_code in lang_codes:
        if lang_code not in header:
            print(f'Error: Language code "{lang_code}" is not found in the language file!')
            sys.exit(1)
        index_lang.append(header.index(lang_code))

    row_header = ['Key']
    row_header.extend(lang_codes)
    if dict_comments:
        row_header.extend(['Type or Unit', 'Comment'])
    if notes:
        row_header.append('Notes')
    output_rows.append(row_header)

    for row in reader:
        key = row[0]
        row_string = [key]
        for index in index_lang:
            cell_content = row[index] if index < len(row) else ''
            if cell_content.startswith(' :'):
                row_string.append(f'"{cell_content}"')
            else:
                row_string.append(cell_content)
        if dict_comments:
            if key in dict_comments:
                row_string.extend(dict_comments[key])
            else:
                row_string.extend(['', ''])
        if notes:
            row_notes = []
            if key in dict_notes_key:
                row_notes.append(dict_notes_key[key])
            for key_keyword in dict_notes_key_keyword:
                if key_keyword in key:
                    row_notes.append(dict_notes_key_keyword[key_keyword])
            for keyword in dict_notes_keyword:
                if keyword in row_string[1].lower():
                    row_notes.append(dict_notes_keyword[keyword])
            row_string.append('; '.join(row_notes))
        output_rows.append(row_string)

with open(output_file, 'w', encoding='utf_8_sig', newline='') as f:
    writer = csv.writer(f)
    writer.writerows(output_rows)

print('Info: Parsing completed!')
